	/*
 * Copyright (c) 2007 Petr Stepan
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup arm32
 * @{
 */
/** @file
    @brief	Exception handlers and exception initialization routines.
 */

#include <arch/exception.h>
#include "aux_print/printf.h"
#include <arch/memstr.h>

#define PREFETCH_OFFSET		0x8
#define BRANCH_OPCODE		0xea000000
#define VALID_BRANCH_MASK	0xff000000

/** Updates specified exception vector to jump to given handler.
*/
static void install_handler (unsigned* handler, unsigned* vector) {
	volatile unsigned vec;
	
	/* calculate relative distance */
	vec = ((unsigned)handler - (unsigned)vector - PREFETCH_OFFSET);
	
	/* word offset */
	vec >>= 2;
	
	/* check if haven't jumped beyond 32 MB */
	if ((vec & VALID_BRANCH_MASK) != 0) {
		/*panic("exception handler beyond 32MB.");*/
		return;
	}
	
	/* make it branch instruction */
	vec |= BRANCH_OPCODE;
	*vector = vec; 
	
	aux_printf("installed  handler to %p, to vector %p\n.", handler, vector);
}

/* TODO add other exception handlers */
static void dummy_exception(){
	asm("bkpt");
}

/** IRQ Exception handler */
static void irq_exception(){
	asm("bkpt");
}

/** Fills exception vectors with appropriate exception handlers.
*/
void install_exception_handlers(void) {
	install_handler((unsigned*)dummy_exception, (unsigned*)EXC_RESET_VEC);
	install_handler((unsigned*)dummy_exception, (unsigned*)EXC_UNDEF_INSTR_VEC);
	install_handler((unsigned*)dummy_exception, (unsigned*)EXC_SWI_VEC);
	install_handler((unsigned*)dummy_exception, (unsigned*)EXC_PREFETCH_ABORT_VEC);
	install_handler((unsigned*)dummy_exception, (unsigned*)EXC_DATA_ABORT_VEC);
	install_handler((unsigned*)irq_exception,   (unsigned*)EXC_IRQ_VEC);
	install_handler((unsigned*)dummy_exception, (unsigned*)EXC_FIQ_VEC);
}

/** Registers exceptions and their handlers to kernel exception dispatcher. */
void exception_init(void){
	/* TODO */
}

/** @}
 */
